/**********************************

	Cel2Shape by Rebecca Ann Heineman (Mac/IIgs/3DO/IBM)

	This will convert a 3DO .Cel file into my generic 3DO cel engine record

**********************************/

#include "cel2shp.h"
#include <stdio.h>

// Pass as bytes to avoid ENDIAN problems

static const Word8 Header[4] = {0,0,0,0x30};

/**********************************

	Scan a 3DO IFF file for a specific record

**********************************/

static void *ScanInput(const char *pTypeName,const Word8 *pInput,WordPtr uLength)
{
	const Word8 *pEnd = pInput+uLength;	// Make final address
	while (pInput<pEnd) {			// At the end?
		Word32 uChunkLength = BigEndian::Load(&reinterpret_cast<const Word32 *>(pInput)[1]);	// Get the length
		if (uChunkLength<8) {	// Invalid length?
			break;
		}
		if (!MemoryCompare(pInput,pTypeName,4)) {	// Does the record match?
			return const_cast<Word8 *>(pInput);				// Return the length
		}
		pInput += uChunkLength;			// Next index
	}
	printf("The record %s was not found!\n",pTypeName);
	return NULL;
}

/**********************************

	Convert a 3DO Raw background into a 15 bit TGA

**********************************/

int BURGER_ANSIAPI main(int argc,const char **argv)
{
	ConsoleApp MyApp(argc,argv);

	if (argc!=3) {			/* Gotta have input and output arguments */
		printf("Usage: cel2shp Infile Outfile\n\n"
			"This program will convert a 3DO cel file into a generic 3DO shape file\n"
			"Copyright 1993-2012 by Rebecca Ann Heineman\n");
		return 10;
	}

	Filename InputName;
	InputName.SetFromNative(argv[1]);
	WordPtr uInputLength;
	Word8 *pInputBuffer = static_cast<Word8 *>(FileManager::LoadFile(&InputName,&uInputLength));
	if (!pInputBuffer) {
		printf("Can't open %s.\n",argv[1]);
		return 10;
	}

	Word RetVal = 10;					// Assume failure
	CCB *pCCB = static_cast<CCB *>(ScanInput("CCB ",pInputBuffer,uInputLength));	// Look for the CCB record
	if (pCCB) {
		// Skip the 12 byte header
		pCCB = reinterpret_cast<CCB *>(reinterpret_cast<Word8 *>(pCCB)+12);
		Word32 uFlags = BigEndian::Load(&pCCB->ccb_Flags);	// Get the flags
		uFlags &= ~0x18000000;	// Make all addresses relative 

		Word32 uPLUTLength = 0;		// Assume that there is no PLUT
		const Word8 *pPLUT = NULL;
		if (uFlags & (1<<23)) {			// Is there a PLUT?
			pPLUT = static_cast<Word8*>(ScanInput("PLUT",pInputBuffer,uInputLength));
			if (!pPLUT) {
				Free(pInputBuffer);		/* Release the memory */
				return RetVal;
			}
			uPLUTLength = BigEndian::Load(&reinterpret_cast<const Word32 *>(pPLUT)[2]);
			uPLUTLength = (uPLUTLength+1)&(~1);
			uPLUTLength *= 2;
			pPLUT += 12;
			// Hard code the relative address
			pCCB->ccb_PLUTPtr = BigEndian::Load(static_cast<Word32>(0x2C));
		}

		const Word8 *pPDATChunk = static_cast<Word8*>(ScanInput("PDAT",pInputBuffer,uInputLength));	/* Get the pixel data */
		if (pPDATChunk) {
			Word32 uPDATLength = BigEndian::Load(&reinterpret_cast<const Word32 *>(pPDATChunk)[1]);
			uPDATLength -= 8;
			pPDATChunk += 8;
			// Create the relative offset
			pCCB->ccb_SourcePtr = BigEndian::Load(0x30+uPLUTLength);
			pCCB->ccb_Flags = BigEndian::Load(uFlags);
			InputName.SetFromNative(argv[2]);
			File Input;
			if (!Input.Open(&InputName,File::WRITEONLY)) {
				if (Input.Write(pCCB,sizeof(CCB)-8)==(sizeof(CCB)-8)) {	/* Save the header */
					if (uPLUTLength) {		/* Any PLUT? */
						Input.Write(pPLUT,uPLUTLength);	/* Save the palette */
					}
					if (Input.Write(pPDATChunk,uPDATLength)==uPDATLength) {		/* Write the shape data */
						RetVal = 0;				/* I did it! */
					}
				}
				Input.Close();
				if (RetVal) {
					printf("Error in saving %s!\n",argv[2]);
				}
			} else {
				printf("Can't open %s!\n",argv[2]);
			}
		}
	}
	Free(pInputBuffer);		/* Release the memory */
	return RetVal;
}
